@@ -19,7 +19,7 @@ import (
 	"fmt"
 	"net/http"
 	"os"
-	"strings"
+	"regexp"
 	"time"
 
 	"github.com/spf13/cobra"
@@ -49,8 +49,7 @@ func newLoginCmd(c *Command) *cobra.Command {
 	cmd := &cobra.Command{
 		Use:   "login [registry]",
 		Short: "log into a CUE registry",
-		Long: `WARNING: THIS COMMAND IS EXPERIMENTAL.
-
+		Long: `
 Log into a CUE registry via the OAuth 2.0 Device Authorization Grant.
 Without an argument, CUE_REGISTRY is used if it points to a single registry.
 
@@ -59,27 +58,21 @@ removing the need for a human to interact with the OAuth device flow.
 
 Once the authorization is successful, a token is stored in a logins.json file
 inside $CUE_CONFIG_DIR; see 'cue help environment'.
-`,
+`[1:],
 		Args: cobra.MaximumNArgs(1),
 		RunE: mkRunE(c, func(cmd *Command, args []string) error {
 			var locResolver modresolve.LocationResolver
+			var err error
 			if len(args) > 0 {
-				var err error
 				locResolver, err = modresolve.ParseCUERegistry(args[0], "")
 				if err != nil {
 					return err
 				}
 			} else {
-				resolver, err := getRegistryResolver()
+				locResolver, err = getRegistryResolver()
 				if err != nil {
 					return err
 				}
-				// We need to do this nil check before we assign to locResolver,
-				// otherwise the interface gets filled with a typed nil, which is not nil.
-				if resolver == nil {
-					return fmt.Errorf("cannot log in when modules are not enabled")
-				}
-				locResolver = resolver
 			}
 			registryHosts := locResolver.AllHosts()
 			if len(registryHosts) > 1 {
@@ -97,9 +90,15 @@ inside $CUE_CONFIG_DIR; see 'cue help environment'.
 			switch tokenStr := flagToken.String(cmd); {
 			case tokenStr == "":
 				// By default, we perform the OAuth 2.0 device flow to obtain a new token.
+				// Note that we refuse to continue if the user set --token="",
+				// because that can be an easy mistake to make via --token=${UNSET_VAR}.
+				if flagToken.IsSet(cmd) {
+					return fmt.Errorf("the --token flag needs a non-empty string")
+				}
 
-				ctx := backgroundContext()
+				ctx := cmd.Context()
 				// Cause the oauth2 logic to log HTTP requests when logging is enabled.
+				// TODO(mvdan): test that these debug logs actually work.
 				ctx = context.WithValue(ctx, oauth2.HTTPClient, &http.Client{
 					Transport: httpTransport(),
 				})
@@ -122,7 +121,7 @@ inside $CUE_CONFIG_DIR; see 'cue help environment'.
 					return fmt.Errorf("cannot obtain the OAuth2 token: %v", err)
 				}
 
-			case strings.HasPrefix(tokenStr, "appv1_"):
+			case tokenRegex.MatchString(tokenStr):
 				// appv1 tokens generated by the CUE registry are valid for one year.
 				// The token string itself does not contain the expiry timestamp,
 				// so we don't store any such timestamp in the logins.json file.
@@ -135,11 +134,7 @@ inside $CUE_CONFIG_DIR; see 'cue help environment'.
 				fallthrough
 
 			default:
-				// For now, reject any unknown token formats.
-				// We could consider handling any arbitrary access tokens just like
-				// we do appv1, only storing the access token and nothing else,
-				// but for now that doesn't seem to be necessary.
-				return fmt.Errorf("unknown token format, expected an appv1_ prefix")
+				return fmt.Errorf("unknown token format, expected a prefix like appv1_")
 			}
 
 			// For consistency, store timestamps in UTC.
@@ -165,3 +160,5 @@ inside $CUE_CONFIG_DIR; see 'cue help environment'.
 const (
 	flagToken flagName = "token"
 )
+
+var tokenRegex = regexp.MustCompile(`^\w{2,10}v\d{1,3}_.+$`)
