@@ -737,44 +737,48 @@ line two.\
 			foo1 = "foo1 value"
 			[[foo.nested1]]
 			foo1_n1 = "foo1_n1 value"
+			not_duplicate = "not a duplicate"
 			[[foo.nested2.deeper]]
 			foo1_n2 = "foo1_n2 value"
 			[[foo]]
 			foo2 = "foo2 value"
 			[[foo.nested1]]
 			foo2_n1 = "foo2_n1 value"
+			not_duplicate = "not a duplicate"
 			[[foo.nested2.deeper]]
 			foo2_n2 = "foo2_n2 value"
 			`,
 		wantCUE: `
-            foo: [
-            	{
-            		foo1: "foo1 value"
-            		nested1: [
-            			{
-            				foo1_n1: "foo1_n1 value"
-            			},
-            		]
-            		nested2: deeper: [
-            			{
-            				foo1_n2: "foo1_n2 value"
-            			},
-            		]
-            	},
-            	{
-            		foo2: "foo2 value"
-            		nested1: [
-            			{
-            				foo2_n1: "foo2_n1 value"
-            			},
-            		]
-            		nested2: deeper: [
-            			{
-            				foo2_n2: "foo2_n2 value"
-            			},
-            		]
-            	},
-            ]
+			foo: [
+				{
+					foo1: "foo1 value"
+					nested1: [
+						{
+							foo1_n1: "foo1_n1 value"
+							not_duplicate: "not a duplicate"
+						},
+					]
+					nested2: deeper: [
+						{
+							foo1_n2: "foo1_n2 value"
+						},
+					]
+				},
+				{
+					foo2: "foo2 value"
+					nested1: [
+						{
+							foo2_n1: "foo2_n1 value"
+							not_duplicate: "not a duplicate"
+						},
+					]
+					nested2: deeper: [
+						{
+							foo2_n2: "foo2_n2 value"
+						},
+					]
+				},
+			]
 			`,
 	}, {
 		name: "RedeclareKeyAsTableArray",
